@@ -123,44 +123,40 @@ void teamdialog_destroy (void)
 void teamdialog_button (GtkWidget *button, gint response, gpointer data)
 {
     GtkEntry *entry = GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (data)));
-    gchar *aux;
-
     button = button; /* so we get no unused parameter warning */
-  
+
     switch (response)
     {
       case GTK_RESPONSE_OK :
       {
-        aux = g_locale_from_utf8 (gtk_entry_get_text (entry), -1, NULL, NULL, NULL);
         gconf_client_set_string (gconf_client, "/apps/gtetrinet/player/team",
                                  gtk_entry_get_text (entry), NULL);
-        tetrinet_changeteam (aux);
-        g_free (aux);
+        tetrinet_changeteam (gtk_entry_get_text(entry));
       }; break;
     }
-    
+
     teamdialog_destroy ();
 }
 
 void teamdialog_new (void)
 {
     GtkWidget *hbox, *widget, *entry;
-    gchar *team_utf8 = g_locale_to_utf8 (team, -1, NULL, NULL, NULL);
+    gchar *team_utf8 = team;
   
     if (team_dialog != NULL)
     {
       gtk_window_present (GTK_WINDOW (team_dialog));
       return;
     }
-  
+
     team_dialog = gtk_dialog_new_with_buttons (_("Change team"),
                                                GTK_WINDOW (app),
                                                GTK_DIALOG_NO_SEPARATOR,
-                                               GTK_STOCK_CANCEL, GTK_RESPONSE_CLOSE,
+                                               GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                                GTK_STOCK_OK, GTK_RESPONSE_OK,
                                                NULL);
     gtk_window_set_skip_taskbar_hint (GTK_WINDOW (team_dialog), TRUE);
-    gtk_dialog_set_default_response (GTK_DIALOG (team_dialog), GTK_RESPONSE_OK);    
+    gtk_dialog_set_default_response (GTK_DIALOG (team_dialog), GTK_RESPONSE_OK);
     gtk_window_set_position (GTK_WINDOW (team_dialog), GTK_WIN_POS_MOUSE);
     gtk_window_set_resizable (GTK_WINDOW (team_dialog), FALSE);
 
@@ -173,11 +169,10 @@ void teamdialog_new (void)
                         team_utf8);
     g_object_set (G_OBJECT (gnome_entry_gtk_entry (GNOME_ENTRY (entry))),
                   "activates_default", TRUE, NULL);
-    g_free (team_utf8);
     gtk_box_pack_start_defaults (GTK_BOX (hbox), entry);
     gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD_SMALL);
     gtk_box_pack_end_defaults (GTK_BOX (GTK_DIALOG (team_dialog)->vbox), hbox);
-    
+
     /* pass the entry in the data pointer */
     g_signal_connect (G_OBJECT(team_dialog), "response",
                         GTK_SIGNAL_FUNC(teamdialog_button), (gpointer)entry);
@@ -198,7 +193,7 @@ static int oldgamemode;
 
 void connectdialog_button (GtkDialog *dialog, gint button)
 {
-    gchar *team_utf8, *nick, *nick1; /* intermediate buffer for recoding purposes */
+    gchar *nick; /* intermediate buffer for recoding purposes */
     const gchar *server1;
     GtkWidget *dialog_error;
 
@@ -217,7 +212,7 @@ void connectdialog_button (GtkDialog *dialog, gint button)
           gtk_widget_destroy (dialog_error);
           return;
         }
-    
+
         spectating = GTK_TOGGLE_BUTTON(spectatorcheck)->active ? TRUE : FALSE;
         if (spectating)
         {
@@ -236,18 +231,13 @@ void connectdialog_button (GtkDialog *dialog, gint button)
           }
         }
         
-        team_utf8 = g_locale_from_utf8 (gtk_entry_get_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry)))),
-                                        -1, NULL, NULL, NULL);
-        
-        GTET_O_STRCPY (team, team_utf8);
+        GTET_O_STRCPY (team, gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry)))));
         
         nick = g_strdup (gtk_entry_get_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (nicknameentry)))));
         g_strstrip (nick); /* we remove leading and trailing whitespaces */
         if (g_utf8_strlen (nick, -1) > 0)
         {
-          nick1 = g_locale_from_utf8 (nick, -1, NULL, NULL, NULL);
-          client_init (server1, nick1);
-          g_free (nick1);
+          client_init (server1, nick);
         }
         else
         {
@@ -266,10 +256,10 @@ void connectdialog_button (GtkDialog *dialog, gint button)
         gconf_client_set_string (gconf_client, "/apps/gtetrinet/player/team",
                                  gtk_entry_get_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry)))),
                                  NULL);
-        g_free (team_utf8);
+
         g_free (nick);
         break;
-    case GTK_RESPONSE_CLOSE:
+    case GTK_RESPONSE_CANCEL:
         gamemode = oldgamemode;
         gtk_widget_destroy (connectdialog);
         break;
@@ -319,7 +309,6 @@ void connectdialog_destroy (void)
 void connectdialog_new (void)
 {
     GtkWidget *widget, *table1, *table2, *frame;
-    gchar *aux;
     /* check if dialog is already displayed */
     if (connecting) 
     {
@@ -335,7 +324,7 @@ void connectdialog_new (void)
     connectdialog = gtk_dialog_new_with_buttons (_("Connect to server"),
                                                  GTK_WINDOW (app),
                                                  GTK_DIALOG_NO_SEPARATOR,
-                                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CLOSE,
+                                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                                  GTK_STOCK_OK, GTK_RESPONSE_OK,
                                                  NULL);
     gtk_dialog_set_default_response (GTK_DIALOG (connectdialog), GTK_RESPONSE_OK);
@@ -432,10 +421,9 @@ void connectdialog_new (void)
     gtk_label_set_mnemonic_widget (GTK_LABEL (widget), nicknameentry);
     g_object_set(G_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(nicknameentry))),
                  "activates_default", TRUE, NULL);
-    aux = g_locale_to_utf8 (nick, -1, NULL, NULL, NULL);
     gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(nicknameentry))),
-                        aux);
-    g_free (aux);
+                        nick);
+    /* g_free (aux);*/
     gtk_widget_show (nicknameentry);
     gtk_table_attach (GTK_TABLE(table2), nicknameentry, 1, 2, 0, 1,
                       GTK_FILL | GTK_EXPAND, 0, 0, 0);
@@ -447,10 +435,9 @@ void connectdialog_new (void)
     gtk_label_set_mnemonic_widget (GTK_LABEL (teamnamelabel), teamnameentry);
     g_object_set(G_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry))),
                  "activates_default", TRUE, NULL);
-    aux = g_locale_to_utf8 (team, -1, NULL, NULL, NULL);
     gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(teamnameentry))),
-                        aux);
-    g_free (aux);
+                        team);
+    /*g_free (aux);*/
     gtk_widget_show (teamnameentry);
     gtk_table_attach (GTK_TABLE(table2), teamnameentry, 1, 2, 1, 2,
                       GTK_FILL | GTK_EXPAND, 0, 0, 0);
@@ -828,7 +815,7 @@ void prefdialog_response (GtkDialog *dialog,
 void prefdialog_new (void)
 {
     GtkWidget *label, *table, *frame, *button, *button1, *widget, *table1, *divider, *notebook;
-    GtkWidget *themelist_scroll, *key_scroll;
+    GtkWidget *themelist_scroll, *key_scroll, *url;
     GtkWidget *channel_list_check;
     GtkListStore *theme_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
     GtkListStore *keys_store = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING);
@@ -847,7 +834,6 @@ void prefdialog_new (void)
                                               GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                                               GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                               NULL);
-    gtk_dialog_set_default_response (GTK_DIALOG (prefdialog), GTK_RESPONSE_CLOSE);
     notebook = gtk_notebook_new ();
     gtk_window_set_resizable (GTK_WINDOW (prefdialog), FALSE);
 
@@ -866,57 +852,51 @@ void prefdialog_new (void)
 
     label = leftlabel_new (_("Select a theme from the list.\n"
                              "Install new themes in ~/.gtetrinet/themes/"));
-    gtk_widget_show (label);
 
     table1 = gtk_table_new (3, 2, FALSE);
     gtk_container_set_border_width (GTK_CONTAINER(table1), GNOME_PAD_SMALL);
     gtk_table_set_row_spacings (GTK_TABLE(table1), 0);
     gtk_table_set_col_spacings (GTK_TABLE(table1), GNOME_PAD_SMALL);
     widget = leftlabel_new (_("Name:"));
-    gtk_widget_show (widget);
     gtk_table_attach (GTK_TABLE(table1), widget, 0, 1, 0, 1,
                       GTK_EXPAND | GTK_FILL, 0, 0, 0);
     widget = leftlabel_new (_("Author:"));
-    gtk_widget_show (widget);
     gtk_table_attach (GTK_TABLE(table1), widget, 0, 1, 1, 2,
                       GTK_EXPAND | GTK_FILL, 0, 0, 0);
     widget = leftlabel_new (_("Description:"));
-    gtk_widget_show (widget);
     gtk_table_attach (GTK_TABLE(table1), widget, 0, 1, 2, 3,
                       GTK_EXPAND | GTK_FILL, 0, 0, 0);
     namelabel = leftlabel_new ("");
-    gtk_widget_show (namelabel);
     gtk_table_attach (GTK_TABLE(table1), namelabel, 1, 2, 0, 1,
                       GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
     authlabel = leftlabel_new ("");
-    gtk_widget_show (authlabel);
     gtk_table_attach (GTK_TABLE(table1), authlabel, 1, 2, 1, 2,
                       GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
     desclabel = leftlabel_new ("");
-    gtk_widget_show (desclabel);
     gtk_table_attach (GTK_TABLE(table1), desclabel, 1, 2, 2, 3,
                       GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-    gtk_widget_show (table1);
 
     frame = gtk_frame_new (_("Selected Theme"));
     gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
     gtk_container_set_border_width (GTK_CONTAINER(frame), GNOME_PAD_SMALL);
     gtk_widget_set_size_request (frame, 240, 100);
-    gtk_widget_show (frame);
     gtk_container_add (GTK_CONTAINER(frame), table1);
-
-    table = gtk_table_new (2, 2, FALSE);
+    
+    table = gtk_table_new (3, 2, FALSE);
     gtk_container_set_border_width (GTK_CONTAINER(table), GNOME_PAD);
     gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
     gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
-    gtk_table_attach (GTK_TABLE(table), themelist_scroll, 0, 1, 0, 2,
+    gtk_table_attach (GTK_TABLE(table), themelist_scroll, 0, 1, 0, 3,
                       GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
     gtk_table_attach (GTK_TABLE(table), label, 1, 2, 0, 1,
                       GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
     gtk_table_attach (GTK_TABLE(table), frame, 1, 2, 1, 2,
                       GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-    gtk_widget_show (table);
-
+    url = gnome_href_new ("http://gtetrinet.sourceforge.net/themes.html", _("Download new themes"));
+    gtk_table_attach (GTK_TABLE(table), url, 1, 2, 2, 3,
+                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_SHRINK, 0, 0);
+    gtk_widget_show_all (table);
+    
     label = gtk_label_new (_("Themes"));
     gtk_widget_show (label);
     gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, label);
